__title__ = 'stoichiograph'
__version__ = '0.1.0'
__license__ = 'MIT'
__author__ = 'Amin Mesbah'
__email__ = 'mesbahamin@gmail.com'
__description__ = 'Spell words with elemental symbols from the periodic table ("He", "Cu", etc).'
